package com.jprofiler.api.agent.probe;

/**
 * A payload info represents a verbose description that can be added added to the current call stack together with
 * associated timing information. Payload infos are created with factory methods in the interceptor context class
 * and can have optional memento objects that help you keep additional state if the payload info is handled
 * in different contexts, such as interception entry and interception exit.
 * <p>
 *     A typical example for payload is the SQL string that is displayed by the built-in JDBC probe.
 * </p>
 *
 * @see InterceptorContext#createPayloadInfo(String)
 * @see InterceptorContext#createPayloadInfo(String, Object)
 */
public interface PayloadInfo {

    /**
     * Returns the description for the payload info.
     * @return the description
     */
    String getDescription();

    /**
     * Sets the description for the payload info.
     * @param description the description
     */
    void setDescription(String description);

    /** Retrieves the memento object.
     * @return the memento object, can be <tt>null</tt>
     */
    Object getMemento();

    /**
     * Sets the memento object. When you are finished with the payload info (for example when you add it to the call tree),
     * you might want to set the memento object to <tt>null</tt>, to prevent possible memory leaks.
     * @param memento the memento object, can be <tt>null</tt>
     */
    void setMemento(Object memento);

    /**
     * Calculates the elapsed time since the creation of the payload and save it in the payload info. This timing information
     * is displayed in the hot spots view of the probe. If the probe annotates data into the call tree, it is also displayed
     * in the call tree view.
     * @return this object
     */
    PayloadInfo calculateTime();

    /**
     * Discards the associated timing information in the payload, so that the displayed time in the JProfiler
     * GUI will be zero.
     * @return this object
     */
    PayloadInfo discardTime();
}
