package com.jprofiler.api.agent.probe;

import java.util.Map;

/**
 * Context object that is passed to probes and provide ways to publish data.
 * An object of this class is passed to telemetry probes in
 * {@link TelemetryProbe#fillTelemetryData(ProbeContext, int[])},
 * the derived {@link InterceptorContext} is passed to the interception methods of
 * {@link InterceptorProbe interceptor probes}.
 * <p>
 *     The <b>help topic on custom probes</b> in the bundled documentation explains how to develop custom probes.
 * </p>
 */
public interface ProbeContext {

    /**
     * Returns a map for storing arbitrary data. This is useful for custom probes that are defined the JProfiler GUI.
     * In that case, you do not write your own classes that could have fields for storing data, but you
     * just provide the necessary scripts.
     * @return the map
     */
    Map getMap();

    /**
     * Creates an event for opening a control object. The probe meta data method
     * {@link ProbeMetaData#controlObjectView(boolean)} has to be called with the argument <tt>true</tt>
     * in the {@link com.jprofiler.api.agent.probe.Probe#getMetaData() meta data configuration method}.
     * <p>
     *     Note that the returned event must be published with {@link #addEvent(ProbeEvent)}, otherwise
     *     the event will be discarded.
     * </p>
     * @param description the description for the event
     * @param controlObject the control object that is being opened
     * @return the probe event
     */
    ProbeEvent createOpenEvent(String description, Object controlObject);

    /**
     * Creates an event for opening a control object with additional control object data.
     * Same as {@link #createOpenEvent(String, Object)} only with additional control object data
     * as defined by calls to {@link ProbeMetaData#addAdditionalControlObjectData(String, DataType, boolean)}.
     * The array must has the same length as the number of calls to that method during the meta data
     * configuration.
     * <p>
     *     Note that the returned event must be published with {@link #addEvent(ProbeEvent)}, otherwise
     *     the event will be discarded.
     * </p>
     * @param description the description for the event
     * @param controlObject the control object that is being opened
     * @param controlObjectData the additional data for the control object
     * @return the probe event
     */
    ProbeEvent createOpenEvent(String description, Object controlObject, Object[] controlObjectData);

    /**
     * Creates an event for closing a control object. The control object must previously have been opened
     * with the {@link #createOpenEvent(String, Object)} or {@link #createOpenEvent(String, Object, Object[])} method.
     * <p>
     *     Note that the returned event must be published with {@link #addEvent(ProbeEvent)}, otherwise
     *     the event will be discarded.
     * </p>
     * @param description the description for the event
     * @param controlObject the control object that is being closed
     * @return the probe event
     */
    ProbeEvent createCloseEvent(String description, Object controlObject);

    /**
     * Creates a custom event. The various custom events are defined with a call to
     * {@link ProbeMetaData#customTypeNames(String[])} in the
     * {@link com.jprofiler.api.agent.probe.Probe#getMetaData() meta data configuration method}.
     * <p>
     *     Note that the returned event must be published with {@link #addEvent(ProbeEvent)}, otherwise
     *     the event will be discarded.
     * </p>
     * @param description the description for the event
     * @param type the type corresponds to the index of the desired custom event in the the call to
     * {@link ProbeMetaData#customTypeNames(String[])}
     * @param controlObject the control object that is associated with this event. May be <tt>null</tt>.
     * @return the probe event
     */
    ProbeEvent createCustomEvent(String description, int type, Object controlObject);

    /**
     * Publishes an event. If single event recording is enabled, you can see the event in the events view of the probe.
     * Otherwise it will only be used for control object management and calculation of sum columns.
     * @param probeEvent the probe event that should be published
     */
    void addEvent(ProbeEvent probeEvent);

    /**
     * Return whether the probe is recording data. The probe will record data on startup unless you called
     * {@link ProbeMetaData#recordOnStartup(boolean)} with the argument <tt>false</tt> in the
     * {@link com.jprofiler.api.agent.probe.Probe#getMetaData() meta data configuration method}. Furthermore, the
     * user can start and stop recording the JProfiler GUI.
     * @return the value
     */
    boolean isRecording();
}
